@@ -462,7 +462,6 @@ namespace MonoTests.System.Web.UI.WebControls
 			DSPoker p = new DSPoker ();
 			p.GetXmlDocument ();
 		}
-#if NET_4_0
 		[Test]
 		public void CacheKeyContext ()
 		{
@@ -474,7 +473,6 @@ namespace MonoTests.System.Web.UI.WebControls
 			xds.CacheKeyContext = "MyKey";
 			Assert.AreEqual ("MyKey", xds.CacheKeyContext, "#A1");
 		}
-#endif
 		[TestFixtureTearDown]
 		public void TearDown ()
 		{
