@@ -56,41 +56,45 @@ gboolean
 mono_threads_core_begin_async_suspend (MonoThreadInfo *info, gboolean interrupt_kernel)
 {
 	kern_return_t ret;
-	gboolean res;
 
 	g_assert (info);
 
-	ret = thread_suspend (info->native_handle);
-	THREADS_SUSPEND_DEBUG ("SUSPEND %p -> %d\n", (void*)info->native_handle, ret);
+
+	do {
+		ret = thread_suspend (info->native_handle);
+	} while (ret == KERN_ABORTED);
+
+	THREADS_SUSPEND_DEBUG ("SUSPEND %p -> %d\n", (gpointer)(gsize)info->native_handle, ret);
 	if (ret != KERN_SUCCESS)
 		return FALSE;
 
 	/* We're in the middle of a self-suspend, resume and register */
 	if (!mono_threads_transition_finish_async_suspend (info)) {
 		mono_threads_add_to_pending_operation_set (info);
-		g_assert (thread_resume (info->native_handle) == KERN_SUCCESS);
-		THREADS_SUSPEND_DEBUG ("FAILSAFE RESUME/1 %p -> %d\n", (void*)info->native_handle, 0);
+		do {
+			ret = thread_resume (info->native_handle);
+		} while (ret == KERN_ABORTED);
+		g_assert (ret == KERN_SUCCESS);
+		THREADS_SUSPEND_DEBUG ("FAILSAFE RESUME/1 %p -> %d\n", (gpointer)(gsize)info->native_handle, 0);
 		//XXX interrupt_kernel doesn't make sense in this case as the target is not in a syscall
 		return TRUE;
 	}
-	res = mono_threads_get_runtime_callbacks ()->
+	info->suspend_can_continue = mono_threads_get_runtime_callbacks ()->
 		thread_state_init_from_handle (&info->thread_saved_state [ASYNC_SUSPEND_STATE_INDEX], info);
-	THREADS_SUSPEND_DEBUG ("thread state %p -> %d\n", (void*)info->native_handle, res);
-	if (res) {
+	THREADS_SUSPEND_DEBUG ("thread state %p -> %d\n", (gpointer)(gsize)info->native_handle, res);
+	if (info->suspend_can_continue) {
 		if (interrupt_kernel)
 			thread_abort (info->native_handle);
 	} else {
-		mono_threads_transition_async_suspend_compensation (info);
-		g_assert (thread_resume (info->native_handle) == KERN_SUCCESS);
-		THREADS_SUSPEND_DEBUG ("FAILSAFE RESUME/2 %p -> %d\n", (void*)info->native_handle, 0);
+		THREADS_SUSPEND_DEBUG ("FAILSAFE RESUME/2 %p -> %d\n", (gpointer)(gsize)info->native_handle, 0);
 	}
-	return res;
+	return info->suspend_can_continue;
 }
 
 gboolean
 mono_threads_core_check_suspend_result (MonoThreadInfo *info)
 {
-	return TRUE;
+	return info->suspend_can_continue;
 }
 
 gboolean
@@ -100,8 +104,8 @@ mono_threads_core_begin_async_resume (MonoThreadInfo *info)
 
 	if (info->async_target) {
 		MonoContext tmp = info->thread_saved_state [ASYNC_SUSPEND_STATE_INDEX].ctx;
-		mach_msg_type_number_t num_state;
-		thread_state_t state;
+		mach_msg_type_number_t num_state, num_fpstate;
+		thread_state_t state, fpstate;
 		ucontext_t uctx;
 		mcontext_t mctx;
 
@@ -110,25 +114,34 @@ mono_threads_core_begin_async_resume (MonoThreadInfo *info)
 		info->async_target = (void (*)(void *)) info->user_data;
 
 		state = (thread_state_t) alloca (mono_mach_arch_get_thread_state_size ());
+		fpstate = (thread_state_t) alloca (mono_mach_arch_get_thread_fpstate_size ());
 		mctx = (mcontext_t) alloca (mono_mach_arch_get_mcontext_size ());
 
-		ret = mono_mach_arch_get_thread_state (info->native_handle, state, &num_state);
+		do {
+			ret = mono_mach_arch_get_thread_states (info->native_handle, state, &num_state, fpstate, &num_fpstate);
+		} while (ret == KERN_ABORTED);
+
 		if (ret != KERN_SUCCESS)
 			return FALSE;
 
-		mono_mach_arch_thread_state_to_mcontext (state, mctx);
+		mono_mach_arch_thread_states_to_mcontext (state, fpstate, mctx);
 		uctx.uc_mcontext = mctx;
 		mono_monoctx_to_sigctx (&tmp, &uctx);
 
-		mono_mach_arch_mcontext_to_thread_state (mctx, state);
+		mono_mach_arch_mcontext_to_thread_states (mctx, state, fpstate);
+
+		do {
+			ret = mono_mach_arch_set_thread_states (info->native_handle, state, num_state, fpstate, num_fpstate);
+		} while (ret == KERN_ABORTED);
 
-		ret = mono_mach_arch_set_thread_state (info->native_handle, state, num_state);
 		if (ret != KERN_SUCCESS)
 			return FALSE;
 	}
 
-	ret = thread_resume (info->native_handle);
-	THREADS_SUSPEND_DEBUG ("RESUME %p -> %d\n", (void*)info->native_handle, ret);
+	do {
+		ret = thread_resume (info->native_handle);
+	} while (ret == KERN_ABORTED);
+	THREADS_SUSPEND_DEBUG ("RESUME %p -> %d\n", (gpointer)(gsize)info->native_handle, ret);
 
 	return ret == KERN_SUCCESS;
 }
